package currencyinput

import (
	"errors"
	"strconv"
	"strings"

	"github.com/charmbracelet/bubbles/textinput"
	tea "github.com/charmbracelet/bubbletea"
)

type Model struct {
	amount textinput.Model
}

func New(prefix string) Model {
	m := Model{
		amount: textinput.New(),
	}

	m.amount.Prompt = prefix
	m.amount.Validate = numberValidator
	m.amount.Placeholder = "00.00"

	return m
}

func numberValidator(s string) error {
	// Should be a number with a maximum of two decimal places
	_, err := strconv.ParseFloat(s, 64)
	if err != nil {
		return err
	}

	decimalIndex := strings.Index(s, ".")
	if decimalIndex == -1 {
		return nil
	}

	decimalPlaces := (len(s) - 1) - decimalIndex
	if decimalPlaces > 2 {
		return errors.New("currency may only have two decimal places")
	}

	return nil
}

func (m Model) Init() tea.Cmd {
	// Sub-components do not have Init functions to call.
	return textinput.Blink
}

func (m Model) Update(msg tea.Msg) (Model, tea.Cmd) {
	var cmd tea.Cmd
	m.amount, cmd = m.amount.Update(msg)
	return m, cmd
}

func (m Model) View() string {
	return m.amount.View()
}

func (m *Model) Focused() bool {
	return m.amount.Focused()
}

func (m *Model) Focus() {
	m.amount.Focus()
}

func (m *Model) Blur() {
	m.amount.Blur()
}
